@@ -823,7 +823,7 @@ void transfer_receive_thread(struct finfo_t *finfo)
     struct timeval min_tstamp;
     int l_file_done_flag, got_naks, cc_len;
     int alldone, found_error, i, len, rcv_status, found_timeout;
-    int do_rewind, do_nextcc, do_halfrate;
+    int do_rewind, do_nextcc, do_halfrate, cc_queued;
     int64_t last_clr;
     double l_adv_grtt;
     uint8_t tos;
@@ -854,17 +854,24 @@ void transfer_receive_thread(struct finfo_t *finfo)
         do_rewind = 0;
         do_nextcc = 0;
         do_halfrate = 0;
+        cc_queued = 0;
         gettimeofday(&now, NULL);
         if ((cc_type == CC_TFMCC) && (cmptimestamp(now, next_cc) > 0)) {
+            if (mux_lock(mux_main)) {
+                glog0(finfo, "Failed to lock mutex in transfer_receive_thread "
+                             "for rewind / feedback");
+                continue;
+            }
+            cc_queued = 1;
             create_cc_list(&cc_body, &cc_len); 
             if (!put_cc_queue(finfo, cc_body, cc_len)) {
                 glog1(finfo, "Couldn't queue up CONG_CTRL: list full!");
                 free(cc_body);
             } else {
+                glog3(finfo, "CONG_CTRL queued");
                 gettimeofday(&next_cc, NULL);
                 add_timeval_d(&next_cc, 1 * l_adv_grtt);
             }
-            glog3(finfo, "CONG_CTRL queued");
 
             // TODO: bypass this check if selected less that 10 RTT ago?
             if (diff_usec(now, last_clr_time) > 1000000 * 4 * l_adv_grtt) {
@@ -883,7 +890,7 @@ void transfer_receive_thread(struct finfo_t *finfo)
         if ((fb_end.tv_sec) && (cmptimestamp(now, fb_end) >= 0)) {
             do_nextcc = 1;
         }
-        if (do_rewind || do_nextcc || do_halfrate) {
+        if ((do_rewind || do_nextcc || do_halfrate) && !cc_queued) {
             if (mux_lock(mux_main)) {
                 glog0(finfo, "Failed to lock mutex in transfer_receive_thread "
                              "for rewind / feedback");
@@ -964,7 +971,7 @@ void transfer_receive_thread(struct finfo_t *finfo)
             cc_seq++;
             glog4(finfo, "Starting feedback round %d", cc_seq);
         }
-        if (do_rewind || do_nextcc || do_halfrate) {
+        if (do_rewind || do_nextcc || do_halfrate || cc_queued) {
             if (mux_unlock(mux_main)) {
                 glog0(finfo, "Failed to unlock mutex in "
                              "transfer_receive_thread for rewind / feedback");
